import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/services.dart';

import 'model/service.dart';

class AppManagement {
  static const MethodChannel _channel = MethodChannel('app_management');
  static const MethodChannel _channel2 = MethodChannel('app_management');
  static bool _isSet = false;

  static Future<String?> get platformVersion async {
    if (!Platform.isAndroid) {
      print('not Supported');
      return null;
    }
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static Future<Memory?> get memoryInfo async {
    if (!Platform.isAndroid) {
      print('not Supported');
      return null;
    }
    final String? mem = await _channel.invokeMethod('getMemoryInfo');
    final parse = json.decode(mem!);
    return Memory(
      available: int.parse(parse["available_mem"]),
      percentage: double.parse(parse["percentage_mem"]),
      total: int.parse(parse["total_mem"]),
    );
  }

  static Future<void> updateApp({Function(int start, int total)? onProgress, Function? onDownloaded, Function? onInstalled, Function? onFailed}) async {
    if (Platform.isAndroid) {
      updateStream = StreamController();
      await _channel.invokeMethod('update');
      _startListener();
      updateStream!.stream.listen((event) {
        if (event == 'DOWNLOADED') {
          if (onDownloaded != null) {
            onDownloaded();
          }
        } else if (event == 'INSTALLED') {
          if (onInstalled != null) {
            onInstalled();
          }
          updateStream!.close();
          updateStream = null;
        } else if (event == 'FAILED') {
          if (onFailed != null) {
            onFailed();
          }
          updateStream!.close();
          updateStream = null;
        } else if (event.toString().contains('|')) {
          final parse = event.toString().split('|');
          if (onProgress != null) {
            try {
              onProgress(int.tryParse(parse[0]) ?? 0, int.tryParse(parse[1]) ?? 0);
            } catch (_) {
              onProgress(0, 0);
            }
          }
        }
      });
    }
  }

  static Future<String?> get killAllApp async {
    if (!Platform.isAndroid) {
      print('not Supported');
      return null;
    }
    return await _channel.invokeMethod('killAll');
  }

  static Future<String?> runInstalledApp(String packageName) async {
    if (!Platform.isAndroid) {
      print('not Supported');
      return null;
    }
    return await _channel.invokeMethod('runAppInPhone', [packageName]);
  }

  static Future<Speed?> getNetworkBandwith() async {
    if (!Platform.isAndroid) {
      print('not Supported');
      return null;
    }
    final mem = await _channel.invokeMethod('getNetworkBandwith');
    final parse = json.decode(mem!);
    return Speed(
      upload: double.parse(parse['up']),
      download: double.parse(parse['down']),
    );
  }

  static Future<List<App>?> getInstalledApps() async {
    if (!Platform.isAndroid) {
      print('not Supported');
      return null;
    }
    final String getApp = await _channel.invokeMethod('getInstalledApp');
    final parseApp = json.decode(getApp);
    List<App> app = [];
    for (var i in parseApp) {
      app.add(App(name: i['name'], packageName: i['package']));
    }
    return app;
  }

  StreamController stream = StreamController();
  static StreamController? updateStream;

  Stream startWhatsappService() {
    _channel.invokeMethod('start_whatsapp_service');
    _startListener(stream: stream);
    return stream.stream;
  }

  static _startListener({stream}) {
    if (!_isSet || stream != null) {
      _isSet = true;
      _channel.setMethodCallHandler((call) async {
        if (call.method == 'whatsapp_message') {
          stream.add(call.arguments);
        } else if (call.method == 'update_status' || call.method == 'update_progress') {
          if (updateStream != null) {
            updateStream!.add(call.arguments);
          }
        }
      });
    }
  }
}
